package org.bench4q.agent.datacollector.impl;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.bench4q.agent.datacollector.DataStatistics;
import org.bench4q.agent.scenario.engine.BehaviorResult;
import org.bench4q.agent.scenario.engine.UnderStatistics;
import org.bench4q.share.models.agent.statistics.BehaviorBriefModel;
import org.bench4q.share.models.agent.statistics.BehaviorStatusCodeResultModel;

public class BehaviorsResultCollector {
	private final Map<Integer, BehaviorResultCollector> behaviorResultMap;

	public BehaviorsResultCollector() {
		this.behaviorResultMap = new HashMap<Integer, BehaviorResultCollector>();
	}

	public void add(BehaviorResult behaviorResult) {
		if (!this.behaviorResultMap.containsKey(behaviorResult.getBehaviorId())) {
			synchronized (this) {
				this.behaviorResultMap.put(
						behaviorResult.getBehaviorId(),
						new BehaviorResultCollector(behaviorResult
								.getBehaviorId()));
			}
		}
		this.behaviorResultMap.get(behaviorResult.getBehaviorId()).add(
				behaviorResult);
	}

	public Object getStatistics(int behaviorId) {
		if (!this.behaviorResultMap.containsKey(behaviorId)) {
			return null;
		}
		BehaviorResultCollector m = this.behaviorResultMap.get(behaviorId);
		return m.getStatistics();
	}

	public static class BehaviorResultCollector implements DataStatistics {
		private int behaviorId;
		private long successCountFromBegin;
		private long failCountFromBegin;
		// statusCode - BehaviorStatusCodeResultCollector
		private final Map<Integer, BehaviorStatusCodeCollector> detailStatusMap;

		public BehaviorResultCollector(int behaviorId) {
			this.behaviorId = behaviorId;
			this.detailStatusMap = new HashMap<Integer, BehaviorStatusCodeCollector>();
		}

		@Override
		public void add(UnderStatistics unit) {
			if (!(unit instanceof BehaviorResult)) {
				return;
			}
			addCore((BehaviorResult) unit);
		}

		public void addCore(BehaviorResult behaviorResult) {
			if (behaviorResult == null
					|| behaviorResult.getBehaviorId() != this.behaviorId) {
				return;
			}
			if (!detailStatusMap.containsKey(behaviorResult.getStatusCode())) {
				guardExist(behaviorResult.getStatusCode(),
						behaviorResult.getContentType());
			}
			this.successCountFromBegin += behaviorResult.getSuccessCount();
			this.failCountFromBegin += behaviorResult.getFailCount();
			BehaviorStatusCodeCollector statusCodeResult = detailStatusMap
					.get(behaviorResult.getStatusCode());
			statusCodeResult.add(behaviorResult);
		}

		private synchronized void guardExist(int statusCode, String contentType) {
			if (!this.detailStatusMap.containsKey(statusCode)) {
				this.detailStatusMap.put(statusCode,
						new BehaviorStatusCodeCollector(this.behaviorId,
								statusCode, contentType));
			}
		}

		public Object getStatistics() {
			BehaviorBriefModel result = new BehaviorBriefModel();
			List<BehaviorStatusCodeResultModel> list = new LinkedList<BehaviorStatusCodeResultModel>();
			for (BehaviorStatusCodeCollector codeResultCollector : this.detailStatusMap
					.values()) {
				list.add(codeResultCollector.getStatistics());
			}
			result.setDetailStatusCodeResultModels(list);
			result.setBehaviorId(this.behaviorId);
			result.setTotalCount(this.successCountFromBegin
					+ this.failCountFromBegin);
			result.setSuccessfulCount(this.successCountFromBegin);
			return result;
		}
	}
}