package org.bench4q.agent.datacollector.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.bench4q.agent.datacollector.DataStatistics;
import org.bench4q.agent.scenario.engine.PageResult;
import org.bench4q.agent.scenario.engine.UnderStatistics;
import org.bench4q.share.models.agent.statistics.AgentPageBriefModel;

public class PagesResultCollector {
	Map<Integer, PageResultCollector> pageBriefMap;

	private Map<Integer, PageResultCollector> getPageBriefMap() {
		return pageBriefMap;
	}

	private void setPageBriefMap(Map<Integer, PageResultCollector> pageBriefMap) {
		this.pageBriefMap = pageBriefMap;
	}

	public PagesResultCollector() {
		this.setPageBriefMap(new HashMap<Integer, PageResultCollector>());
	}

	public void add(PageResult pageResult) {
		if (pageResult == null || pageResult.getPageId() < 0) {
			return;
		}
		PageResultCollector pageBrief = this.getPageBriefMap().get(
				pageResult.getPageId());
		if (!this.getPageBriefMap().containsKey(pageResult.getPageId())) {
			pageBrief = guardTheValueOfThePageIdExists(pageResult.getPageId());
		}
		pageBrief.addCore(pageResult);
	}

	private synchronized PageResultCollector guardTheValueOfThePageIdExists(
			int pageId) {
		if (!this.getPageBriefMap().containsKey(pageId)) {
			this.getPageBriefMap().put(pageId, new PageResultCollector(pageId));
		}
		return this.getPageBriefMap().get(pageId);
	}

	public Object getPageBriefStatistics(int pageId) {
		PageResultCollector pageBrief = guardTheValueOfThePageIdExists(pageId);
		return pageBrief.getStatistics();
	}

	public class PageResultCollector implements DataStatistics {
		private long lastSampleTime;
		private long countThisTime;
		private long totalResponseTimeThisTime;
		private long maxResponseTimeFromBegin;
		private long minResponseTimeFromBegin;
		private long countFromBegin;
		private long latestTimeResponseTime;
		private final int pageId;

		public PageResultCollector(int pageId) {
			resetTemperatyField();
			this.maxResponseTimeFromBegin = Long.MIN_VALUE;
			this.minResponseTimeFromBegin = Long.MAX_VALUE;
			this.countFromBegin = 0;
			this.latestTimeResponseTime = 0;
			this.pageId = pageId;
		}

		public void resetTemperatyField() {
			this.lastSampleTime = new Date().getTime();
			this.countThisTime = 0;
			this.totalResponseTimeThisTime = 0;
		}

		public void add(UnderStatistics unit) {
			if (!(unit instanceof PageResult)) {
				return;
			}
			addCore((PageResult) unit);
		}

		public void addCore(PageResult pageResult) {
			if (pageResult == null || pageResult.getPageId() != this.pageId) {
				return;
			}
			this.countThisTime++;
			this.countFromBegin++;
			this.totalResponseTimeThisTime += pageResult.getExecuteRange();
			this.latestTimeResponseTime = pageResult.getExecuteRange();
			if (pageResult.getExecuteRange() > this.maxResponseTimeFromBegin) {
				this.maxResponseTimeFromBegin = pageResult.getExecuteRange();
			}
			if (pageResult.getExecuteRange() < this.minResponseTimeFromBegin) {
				this.minResponseTimeFromBegin = pageResult.getExecuteRange();
			}
		}

		public Object getStatistics() {
			AgentPageBriefModel result = new AgentPageBriefModel();
			result.setCountFromBegin(countFromBegin);
			result.setCountThisTime(countThisTime);
			result.setMaxResponseTimeFromBegin(maxResponseTimeFromBegin);
			result.setMinResponseTimeFromBegin(minResponseTimeFromBegin);
			result.setTotalResponseTimeThisTime(totalResponseTimeThisTime);
			result.setPageId(pageId);
			long nowTime = new Date().getTime();
			result.setTimeFrame(nowTime - lastSampleTime);
			result.setLatestResponseTime(latestTimeResponseTime);
			this.resetTemperatyField();
			return result;
		}

	}
}
