package org.bench4q.recorder.httpcapture.generator;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.nio.charset.Charset;

import org.apache.log4j.Logger;
import org.bench4q.recorder.httpcapture.ResponseModel;

public class ResponseParser {
	private Logger logger = Logger.getLogger(ResponseParser.class);
	private byte[] response;
	private ResponseHeader responseHeader;
	private String responseBody;

	private byte[] getResponse() {
		return response;
	}

	private void setResponse(byte[] response) {
		this.response = response;
	}

	public ResponseHeader getResponseHeader() {
		return responseHeader;
	}

	private void setResponseHeader(ResponseHeader responseHeader) {
		this.responseHeader = responseHeader;
	}

	public String getResponseBody() {
		return responseBody;
	}

	private void setResponseBody(String responseBody) {
		this.responseBody = responseBody;
	}

	public ResponseParser(ResponseModel response) {
		this.setResponse(response.getResponse());
		this.setResponseHeader(parseResponseHeader(response.getConn()));
		if (this.getResponseHeader().isValidResponseHeader()
				&& this.getResponseHeader().isHtmlContentType()) {
			this.parseResponseBody();

		}
	}

	private ResponseHeader parseResponseHeader(
			HttpURLConnection httpURLConnection) {
		ResponseHeader result = new ResponseHeader();
		if (httpURLConnection == null)
			return result;
		result.setContentEncoding(httpURLConnection.getContentEncoding());
		result.setContentLength(httpURLConnection.getContentLength());
		try {
			result.setRespCode(httpURLConnection.getResponseCode());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		result.setTransferEncoding(parseTransferEncoding(httpURLConnection));
		result.setCharset(parseCharset(httpURLConnection));
		result.setContentType(parseContentType(httpURLConnection));
		return result;
	}

	private String parseCharset(HttpURLConnection httpURLConnection) {

		String ret = null;
		String value = httpURLConnection.getHeaderField("Content-Type");
		int pos;
		if (value != null && (pos = value.indexOf("charset=")) != -1) {
			ret = value.substring(pos + 8);
		}

		return ret;
	}

	private String parseContentType(HttpURLConnection httpURLConnection) {

		String ret = null;
		String value = httpURLConnection.getHeaderField("Content-Type");
		int pos;
		if (value != null) {
			if ((pos = value.indexOf(";")) != -1) {
				ret = value.substring(0, pos);
			} else {

				ret = value;
			}
		}

		return ret;
	}

	private String parseTransferEncoding(HttpURLConnection httpURLConnection) {

		return httpURLConnection.getHeaderField("Transfer-Encoding");
	}

	private void parseResponseBody() {
		ContentDecoder contentDecoder = ContentDecoder.createDecoder(this
				.getResponseHeader().getContentEncoding());
		byte[] contentBodyAfterDecoded = null;

		contentBodyAfterDecoded = contentDecoder
				.decodeContent(new ByteArrayInputStream(this.getResponse(), 0,
						this.getResponse().length));
		if (contentBodyAfterDecoded == null) {
			return;
		}
		Charset charset = null;
		try {
			charset = Charset.forName(this.getResponseHeader().getCharset());
		} catch (Exception e) {
			charset = getCharsetFromContent(contentBodyAfterDecoded);
			if (charset == null) {
				charset = Charset.forName("utf-8");
			}
			this.getResponseHeader().setContentCharset(charset);
			logger.error(e, e);
		}
		this.setResponseBody(new String(contentBodyAfterDecoded, charset));
	}

	private Charset getCharsetFromContent(byte[] content) {
		String contentString = new String(content,
				Charset.forName("iso-8859-1")).toLowerCase();
		int pos = contentString.indexOf("charset=");
		if (pos == -1)
			return null;
		pos += 8;
		char tempChar = contentString.charAt(pos);
		StringBuilder sBuilder = new StringBuilder();
		if (tempChar == '"') {
			pos++;
		}
		while ((tempChar = contentString.charAt(pos)) != '"') {
			sBuilder.append(tempChar);
			pos++;
		}
		try {
			return Charset.forName(sBuilder.toString());
		} catch (Exception e) {
			return null;
		}
	}

	public byte[] encodeContent(String responseBody) {
		ContentEncoder contentEncoder = ContentEncoder.createEncoder(this
				.getResponseHeader().getContentEncoding());
		Charset charset = null;
		try {
			charset = Charset.forName(this.getResponseHeader().getCharset());
		} catch (Exception e) {
			charset = this.getResponseHeader().getContentCharset();
			if (charset == null)
				charset = Charset.forName("utf-8");
			logger.error(e, e);
		}
		byte[] responseBytes = responseBody.getBytes(charset);
		responseBytes = contentEncoder.encoderContent(responseBytes);

		return responseBytes;
	}
}
